# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class RawResponse(object):
    PATTERN = ("HTTP/{proto_major}.{proto_minor} "
               "{int_status} {verbose_status}\r\n"
               "{headers}"
               "{response_data}")

    def __init__(self, http_proto_version, status_code, verbose_status,
                 http_headers=None, response_data=None):
        """Sets up raw response
        :param http_proto_version: HTTP protocol version
        :type http_proto_version: tuple
        :param status_code: HTTP response code
        :type status_code: int
        :param verbose_status: HTTP response status
        :type verbose_status: str
        :param http_headers: HTTP response headers
        :type http_headers: dict
        :param response_data: string representation of data
        :type response_data: str
        """

        http_headers = http_headers if http_headers else {}
        self.http_proto = http_proto_version
        self.int_status = status_code
        self.verbose_status = verbose_status
        self.response_data, content_len = self.__encode_data(response_data)
        if self.response_data:
            http_headers.update({"Content-Length": content_len})
        self.headers = self.__encode_headers(http_headers)

    def __encode_headers(self, headers):
        if headers:
            result = ""
            for hk, hv in headers.items():
                result += "{}: {}\r\n".format(hk, hv)
            return result + "\r\n"

    def __encode_data(self, data):
        return data + "\r\n" if data else "", len(data) if data else 0

    def dump(self):
        format_map = {
            "proto_major": self.http_proto[0],
            "proto_minor": self.http_proto[1],
            "int_status": self.int_status,
            "verbose_status": self.verbose_status,
            "headers": self.headers,
            "response_data": self.response_data,
        }
        return self.PATTERN.format(**format_map)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class RawRequest(object):

    def __init__(self, raw_request):
        self.raw = raw_request

    def parse_raw_request(self):
        """Parses raw HTTP request that contains:
         - method
         - route + query
         - headers
         - protocol version
         Optionally:
         - data

         Raw HTTP request may have next look:

            GET /v1/apps?something=something&etc=etc HTTP/1.1
            Host: localhost:8080
            Content-Length: 5
            Content-Type: application/x-www-form-urlencoded
            User-Agent: curl/7.51.0

            hello

        Each new line define by set of special characters:

            \n
            \r

        and combination is:

            \r\n

        :return: tuple of HTTP method, HTTP URL, HTTP query parameters,
        HTTP headers, HTTP proto version, HTTP raw data
        :rtype: tuple
        """

        def get_parts_by_identifier(raw_req, condition):
            return [part for part in raw_req.split("\r\n") if condition(part)]

        def setup_headers(raw_headers):
            result = {}
            for h in raw_headers:
                hk, hv = h.split(":", maxsplit=1)
                result.update({hk: hv.lstrip()})
            return result

        rest_data = get_parts_by_identifier(
            self.raw,
            lambda x: x.find(":") < 0 and x != "" and "\n" not in x)

        headers = setup_headers(get_parts_by_identifier(
            self.raw, lambda y: y.find(":") > 0 and y != ""))

        data = ''
        dict_params = {}
        try:
            if int(headers.get("Content-Length", 0)) > 0:
                # if data sent, it will be presented
                # at the end of the raw request
                raw_data = self.raw.rsplit("\r\n", maxsplit=1).pop()
                data = raw_data[:-1]
                # it may appear that data contains `:` chars (JSON)
                # in this case data will be appended to headers,
                # that why it must be dropped off
                headers = setup_headers(get_parts_by_identifier(
                    self.raw[:-(len(data) + 2)],
                    lambda y: y.find(":") > 0 and y != ""))

            method, route_query, version = rest_data.pop().split(" ")
            major_minor = version.replace("HTTP/", "").split(".")
            if len(major_minor) > 1:
                major, minor = major_minor
            else:
                major, minor = major_minor.pop(), "0"
            splitted_route_query = route_query.split("?")
            if len(splitted_route_query) == 2:
                url, params = splitted_route_query
                for pair in params.split("&"):
                    k, v = pair.split("=")
                    dict_params.update({k: v})
            else:
                url = splitted_route_query.pop()

            return method, url, dict_params, headers, (major, minor), data

        except (Exception, IndexError) as ex:
            raise Exception("Malformed HTTP method, route, version: {}. "
                            "Original error: {}".format(rest_data, str(ex)))



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import setuptools


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setuptools.setup(
    name='hotfn',
    version='0.0.1',
    description='Library to provide HTTP over STDIN/STDOUT',
    long_description=read('README.md'),
    url='https://hotfn.readthedocs.org',
    author='Denis Makogon',
    author_email='lildee1991@gmail.com',
    packages=setuptools.find_packages(exclude=["hotfn.tests", ]),
    install_requires=[],
    license='License :: OSI Approved :: Apache Software License',
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Developers',
        'Environment :: No Input/Output (Daemon)',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: '
        'Libraries :: Python Modules',
        'Topic :: System :: Distributed Computing',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    keywords=['functions', 'lambdas', 'python API'],
    platforms=['Linux', 'Mac OS-X', 'Unix'],
    tests_require=[
        "flake8==2.5.0",
        "hacking<0.11,>=0.10.0",
        "testtools>=1.4.0",
        "pytest-cov==2.4.0",
        "pytest-cov",
    ],
    zip_safe=True,
)
